/* **COPYRIGHT******************************************************************
    INTEL CONFIDENTIAL
    Copyright (C) 2017 Intel Corporation
    Copyright (C), 1994-2004 Aware Inc. All Rights Reserved.
******************************************************************COPYRIGHT** */
/* **DISCLAIMER*****************************************************************
    The source code contained or described herein and all documents related
    to the source code ("Material") are owned by Intel Corporation or its
    suppliers or licensors. Title to the Material remains with Intel
    Corporation or its suppliers and licensors. The Material may contain
    trade secrets and proprietary and confidential information of Intel
    Corporation and its suppliers and licensors, and is protected by
    worldwide copyright and trade secret laws and treaty provisions. No part
    of the Material may be used, copied, reproduced, modified, published,
    uploaded, posted, transmitted, distributed, or disclosed in any way
    without Intel's prior express written permission.

    No license under any patent, copyright, trade secret or other
    intellectual property right is granted to or conferred upon you by
    disclosure or delivery of the Materials, either expressly, by
    implication, inducement, estoppel or otherwise. Any license under
    such intellectual property rights must be express and approved by
    Intel in writing.
*****************************************************************DISCLAIMER** */
/*
*-------------------------------------------------------------------------------
*
*   Aware DMT Technology. Proprietary and Confidential.
*
*   40 Middlesex Turnpike, Bedford, MA 01730-1413
*   Phone (781) 276 - 4000
*   Fax   (781) 276 - 4001
*
*   filename: qt_cocomo_interface.h
*
*   This file Specifies the Cocomo to Iridia-QT interface.  This interface
*  includes the register read/write functions, core reset functions, BM
*  intialization functions, and any other cocomo/BM only functions/variables
*  that are required to run a HW Engine simulation.  These functions are NOT
*  required by, and unavailable for, actual HW.
*
*-------------------------------------------------------------------------------
*/

#ifndef _QT_COCOMO_INTERFACE_H
#define _QT_COCOMO_INTERFACE_H


#include "qt_typedef.h"
#include "qt_io_typedef.h"

//=============================================================================
// Iridia-QT BM clocking functions
//=============================================================================
void IRI_QT_Clock(void);
void IRI_QT_Clock_Tx_Frame(uint8 uc_port);
void IRI_QT_Clock_Rx_Frame(uint8 uc_port);


//=============================================================================
// Iridia-QT BM initialization functions
//=============================================================================
void IRI_QT_InitializeBM(void);

//=============================================================================
// Iridia-QT BM reset functions
//=============================================================================
void IRI_QT_Reset(void);
void IRI_QT_SetportMode(uint8 uc_port_mode);

//=============================================================================
// Iridia-QT functions called from other BMs.
//=============================================================================
//int32 IRI_QTP_GetToneReorderingTableAddress(uint16 **pus_BaseAddr,uint8 uc_port);
int32 IRI_QT_GetActiveBGTNTableAddress(uint32 **pus_BaseAddr, int32 s_Path,uint8 uc_port);

//int32 IRI_QT_GetGainScaleTableAddress(uint16 **pus_BaseAddr, uint8 uc_port);
//int32 IRI_QT_GetBitAllocationTableAddress(uint8 **puc_BaseAddr, uint8 uc_port);
//int32 IRI_QT_GetExtendedGainTableAddress(uint8 **puc_BaseAddr, uint8 uc_port);
int32 IRI_QT_GetActiveEGTTableAddress(uint8 **puc_BaseAddr, uint8 uc_port);






//int32 IRI_QT_TX_GetTssiTableAddress(uint16 **pusa_Tssi,uint8 uc_port);
int32 IRI_QT_GetSFDQExponentTableAddress(uint8 **puc_BaseAddr, uint8 uc_port);
int32 IRI_QT_GetSFDQMantissaTableAddress(int16 **ps_BaseAddr,uint8 uc_port);
int32 IRI_QT_GetMFDQFBMantissaTableAddress(int16 **ps_BaseAddr,uint8 uc_port);
int32 IRI_QT_GetMFDQFFMantissaTableAddress(int16 **ps_BaseAddr,uint8 uc_port);
int32 IRI_QT_GetMFDQFBExponentTableAddress(uint32 **pul_BaseAddr,uint8 uc_port);
int32 IRI_QT_GetMFDQFBToneTableAddress(uint32 **pul_BaseAddr,uint8 uc_port);
int32 IRI_QT_GetLSWNoiseAccmrAddress(int16 **ps_BaseAddr,uint8 uc_port);
int32 IRI_QT_GetMSWNoiseAccmrAddress(int16 **ps_BaseAddr,uint8 uc_port);

int32 IRI_QT_GetMFDQFinalDecisionTableAddress(uint32 **pula_MfdqFinalDecisionTbl,uint8 uc_port);


int32 IRI_QT_GetTxConstellationGainsTable(uint32 *gula_TxConstellationGains);
uint32 GetTxLookuptableOffset(uint32 ul_offset,uint8 uc_ByteEnable,uint8 uc_port);

//wrapper functions
void QT_ZEP_ILV_GetWordFromTxDTB(int16 s_lp, uint32 *pul_word, uint8 uc_port);
int32 IRI_QT_FTB_ReadRxFFT(int32 *pla_Rx_OutputArray, uint8 uc_port);
int32 IRI_QT_FTB_ReadRxFFT_returnSize(int32 *pla_Rx_OutputArray, int32 *s_RxFFTNumTones, uint8 uc_port);
void QT_IRI_FTB_TxWriteIFFT(int32 realValue,int32 imagValue, uint16 toneValue,uint8 uc_port, uint8 skip);

void QT_ZEP_ILV_PutWordToRxDTB(int16 s_lpath, uint32 ul_OutWord, uint8 uc_port);

void IRI_QT_TxGetDTSize(uint8 port, uint16 *pusa_DTSize);
void IRI_QT_RxGetDTSize(uint8 port, uint16 *pusa_DTSize);

void IRI_QT_IIBRAM_Write(uint8 uc_port, uint32 offset, uint32 data, uint32 bit_enable);

uint8 IRI_QT_GetTxBATErr(uint8 port);
uint8 IRI_QT_GetRxBATErr(uint8 port);

//=============================================================================
// Iridia-QT functions for reading/writing RAM blocks and registers
//=============================================================================
int32 IRI_QT_WriteWord(uint32 addr, uint32 ul_data, uint8 uc_ByteEnable,uint8 uc_port);
int32 IRI_QT_ReadWord(uint32 addr, uint32 *pul_data, uint8 uc_ByteEnable,uint8 uc_port);




#endif // _QT_COCOMO_INTERFACE_H
